"use strict";
/**
 * Created by paul on 7/25/17.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.abs = exports.toFixed = exports.log10 = exports.max = exports.min = exports.eq = exports.lte = exports.gte = exports.lt = exports.gt = exports.div = exports.mul = exports.sub = exports.add = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
// const MAX_DECIMALS = 10
function isHex(x) {
    if (x.startsWith('0x') ||
        x.startsWith('-0x') ||
        x.toLowerCase().includes('a') ||
        x.toLowerCase().includes('b') ||
        x.toLowerCase().includes('c') ||
        x.toLowerCase().includes('d') ||
        x.toLowerCase().includes('e') ||
        x.toLowerCase().includes('f')) {
        return true;
    }
    else {
        return false;
    }
}
function cropHex(x) {
    return x.replace('0x', '');
}
function addZeros(val, numZeros) {
    let out = val;
    for (let n = 0; n < numZeros; n++) {
        out += '0';
    }
    return out;
}
// Remove starting and trailing zeros and decimal
function trimEnd(val) {
    // Remove starting zeros if there are any
    let out = val.replace(/^0+/, '');
    out = out.replace(/^\.+/, '0.');
    if (out.includes('.')) {
        // Remove trailing zeros
        out = out.replace(/0+$/, '');
        // Remove trailing "." if there is one
        out = out.replace(/\.+$/, '');
        if (out === '') {
            out = '0';
        }
    }
    if (out === '') {
        out = '0';
    }
    return out;
}
function addDecimal(x, shift) {
    if (shift === 0)
        return x;
    let isNegative = false;
    if (x.slice(0, 1) === '-') {
        isNegative = true;
        x = x.slice(1);
    }
    let out;
    if (shift > x.length) {
        out = '0.' + addZeros('', shift - x.length) + x;
    }
    else {
        out =
            x.substr(0, x.length - shift) + '.' + x.substr(x.length - shift, x.length);
    }
    out = trimEnd(out);
    if (isNegative) {
        out = `-${out}`;
    }
    return out;
}
// Takes two floating point (base 10) numbers and finds the multiplier needed to make them both
// operable as a integer
function floatShifts(xStart, yStart, moreShift) {
    let x = xStart;
    let y = yStart;
    validate(x, y);
    let xPos = x.indexOf('.');
    let yPos = y.indexOf('.');
    const xHex = isHex(x);
    const yHex = isHex(y);
    if (xPos !== -1) {
        // Remove trailing zeros
        x = trimEnd(x);
        xPos = x.indexOf('.');
    }
    if (yPos !== -1) {
        // Remove trailing zeros
        y = trimEnd(y);
        yPos = y.indexOf('.');
    }
    if (xPos !== -1 || yPos !== -1 || typeof moreShift === 'number') {
        if (xHex || yHex) {
            throw new Error('Cannot operate on base16 float values');
        }
        let xShift = 0;
        let yShift = 0;
        if (xPos !== -1) {
            xShift = x.length - xPos - 1;
        }
        if (yPos !== -1) {
            yShift = y.length - yPos - 1;
        }
        const shift = xShift > yShift ? xShift : yShift;
        let moreS = 0;
        if (typeof moreShift === 'number') {
            moreS = moreShift;
        }
        x = addZeros(x.replace('.', ''), shift + moreS - xShift);
        y = addZeros(y.replace('.', ''), shift - yShift);
        const out = { x, y, shift };
        return out;
    }
    else {
        // Both x and y are int and need no float conversion
        const out = {
            x,
            y,
            shift: 0,
        };
        return out;
    }
}
function validate(...args) {
    for (const arg of args) {
        if (arg.split('.').length - 1 > 1) {
            throw new Error('Invalid number: more than one decimal point');
        }
        if (arg.split('-').length - 1 > 1) {
            throw new Error('Invalid number: more than one negative sign');
        }
    }
}
function add(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.add(yBN).toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : '0x' + out;
}
exports.add = add;
function mul(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.mul(yBN).toString(base);
    out = addDecimal(out, shift * 2);
    return base === 10 ? out : '0x' + out;
}
exports.mul = mul;
function sub(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.sub(yBN).toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : '0x' + out;
}
exports.sub = sub;
function div(x1, y1, precision = 0, base = 10) {
    if (base !== 10 && precision > 0) {
        throw new Error('Cannot operate on floating point hex values');
    }
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y } = floatShifts(x1, y1, precision);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.div(yBN).toString(base);
    out = addDecimal(out, precision);
    return base === 10 ? out : '0x' + out;
}
exports.div = div;
function lt(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.lt(yBN);
}
exports.lt = lt;
function lte(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.lte(yBN);
}
exports.lte = lte;
function gt(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.gt(yBN);
}
exports.gt = gt;
function gte(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.gte(yBN);
}
exports.gte = gte;
function eq(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.eq(yBN);
}
exports.eq = eq;
function min(x1, y1, base = 10) {
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out;
    if (xBN.lte(yBN)) {
        out = xBN.toString(base);
    }
    else {
        out = yBN.toString(base);
    }
    out = addDecimal(out, shift);
    return base === 10 ? out : '0x' + out;
}
exports.min = min;
function abs(x1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, shift } = floatShifts(x1, '0');
    const xBase = isHex(x1) ? 16 : 10;
    x = cropHex(x);
    const xBN = new bn_js_1.default(x, xBase);
    let out = xBN.abs().toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : '0x' + out;
}
exports.abs = abs;
function max(x1, y1, base = 10) {
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out;
    if (xBN.gte(yBN)) {
        out = xBN.toString(base);
    }
    else {
        out = yBN.toString(base);
    }
    out = addDecimal(out, shift);
    return base === 10 ? out : '0x' + out;
}
exports.max = max;
function toFixed(x1, minPrecision = 2, maxPrecision = 8) {
    validate(x1);
    let negative = false;
    let out = '';
    let x = x1;
    if (x.includes('-')) {
        negative = true;
        // Remove any leading '-' signs
        x = x.replace(/^-+/, '');
    }
    x = trimEnd(x);
    // Number of decimal places number has
    const decimalPos = x.indexOf('.');
    if (decimalPos === -1) {
        out = x + '.' + addZeros('', minPrecision);
    }
    else {
        const numDecimals = x.length - decimalPos - 1;
        if (numDecimals > maxPrecision) {
            out = x.substr(0, x.length - (numDecimals - maxPrecision));
        }
        else if (numDecimals < minPrecision) {
            out = x + addZeros('', minPrecision - numDecimals);
        }
        else {
            out = x;
        }
    }
    // Remove trailing "." if there is one
    out = out.replace(/\.+$/, '');
    if (negative) {
        out = '-' + out;
    }
    return out;
}
exports.toFixed = toFixed;
function log10(x) {
    if (!(x.match(/^[0-1]+$/g) !== null)) {
        throw new Error('InvalidLogInputValue: Must be a power of 10');
    }
    if (!x.startsWith('1')) {
        throw new Error('InvalidLogInputValue: Must not have leading zeros');
    }
    if ((x.match(/1/g) || []).length > 1) {
        throw new Error('InvalidLogInputValue: Must be power of 10.');
    }
    return (x.match(/0/g) || []).length;
}
exports.log10 = log10;
