"use strict";
/**
 * Created by paul on 7/25/17.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.log10 = exports.toFixed = exports.toBns = exports.round = exports.ceil = exports.floor = exports.max = exports.abs = exports.min = exports.eq = exports.gte = exports.gt = exports.lte = exports.lt = exports.div = exports.sub = exports.mul = exports.add = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const SCI_NOTATION_REGEX = /^(-?\d*\.?\d*)e((?:\+|-)?\d+)$/;
// -----------------------------------------------------------------------------
// Public
// -----------------------------------------------------------------------------
function add(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.add(yBN).toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.add = add;
function mul(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.mul(yBN).toString(base);
    out = addDecimal(out, shift * 2);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.mul = mul;
function sub(x1, y1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.sub(yBN).toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.sub = sub;
function div(x1, y1, precision = 0, base = 10) {
    if (base !== 10 && precision > 0) {
        throw new Error('Cannot operate on floating point hex values');
    }
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, y } = floatShifts(x1, y1, precision);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out = xBN.div(yBN).toString(base);
    out = addDecimal(out, precision);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.div = div;
function lt(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.lt(yBN);
}
exports.lt = lt;
function lte(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.lte(yBN);
}
exports.lte = lte;
function gt(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.gt(yBN);
}
exports.gt = gt;
function gte(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.gte(yBN);
}
exports.gte = gte;
function eq(x1, y1) {
    let { x, y } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    return xBN.eq(yBN);
}
exports.eq = eq;
function min(x1, y1, base = 10) {
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out;
    if (xBN.lte(yBN)) {
        out = xBN.toString(base);
    }
    else {
        out = yBN.toString(base);
    }
    out = addDecimal(out, shift);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.min = min;
function abs(x1, base = 10) {
    if (base !== 10 && base !== 16)
        throw new Error('Unsupported base');
    let { x, shift } = floatShifts(x1, '0');
    const xBase = isHex(x1) ? 16 : 10;
    x = cropHex(x);
    const xBN = new bn_js_1.default(x, xBase);
    let out = xBN.abs().toString(base);
    out = addDecimal(out, shift);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.abs = abs;
function max(x1, y1, base = 10) {
    let { x, y, shift } = floatShifts(x1, y1);
    const xBase = isHex(x) ? 16 : 10;
    const yBase = isHex(y) ? 16 : 10;
    x = cropHex(x);
    y = cropHex(y);
    const xBN = new bn_js_1.default(x, xBase);
    const yBN = new bn_js_1.default(y, yBase);
    let out;
    if (xBN.gte(yBN)) {
        out = xBN.toString(base);
    }
    else {
        out = yBN.toString(base);
    }
    out = addDecimal(out, shift);
    return base === 10 ? out : out.replace(/^(-)?/, '$10x');
}
exports.max = max;
exports.floor = (x1, precision) => precisionAdjust('floor', x1, precision);
exports.ceil = (x1, precision) => precisionAdjust('ceil', x1, precision);
exports.round = (x1, precision) => precisionAdjust('round', x1, precision);
function toBns(n) {
    let out = typeof n === 'number' ? n.toString() : n.replace(/^\s+|\s+$/g, '');
    if (out === '' || out === '.') {
        out = '0';
    }
    // Handle scientific notation
    const match = out.match(SCI_NOTATION_REGEX);
    if (match != null) {
        const base = match[1];
        const exponent = parseInt(match[2]);
        out = sciNotation(base, exponent);
    }
    validate(out);
    return out;
}
exports.toBns = toBns;
function toFixed(x1, minPrecision = 2, maxPrecision = 8) {
    let x = toBns(x1);
    let negative = false;
    let out = '';
    if (x.includes('-')) {
        negative = true;
        // Remove any leading '-' signs
        x = x.replace(/^-+/, '');
    }
    x = trimEnd(x);
    // Number of decimal places number has
    const decimalPos = x.indexOf('.');
    if (decimalPos === -1) {
        out = x + '.' + addZeros('', minPrecision);
    }
    else {
        const numDecimals = x.length - decimalPos - 1;
        if (numDecimals > maxPrecision) {
            out = x.substr(0, x.length - (numDecimals - maxPrecision));
        }
        else if (numDecimals < minPrecision) {
            out = x + addZeros('', minPrecision - numDecimals);
        }
        else {
            out = x;
        }
    }
    // Remove trailing "." if there is one
    out = out.replace(/\.+$/, '');
    if (negative && out !== '0') {
        out = '-' + out;
    }
    return out;
}
exports.toFixed = toFixed;
function log10(x) {
    if (!(x.match(/^[0-1]+$/g) !== null)) {
        throw new Error('InvalidLogInputValue: Must be a power of 10');
    }
    if (!x.startsWith('1')) {
        throw new Error('InvalidLogInputValue: Must not have leading zeros');
    }
    if ((x.match(/1/g) || []).length > 1) {
        throw new Error('InvalidLogInputValue: Must be power of 10.');
    }
    return (x.match(/0/g) || []).length;
}
exports.log10 = log10;
// -----------------------------------------------------------------------------
// Private
// -----------------------------------------------------------------------------
function addDecimal(x, shift) {
    if (shift === 0)
        return x;
    let isNegative = false;
    if (x.slice(0, 1) === '-') {
        isNegative = true;
        x = x.slice(1);
    }
    let out;
    if (shift > x.length) {
        out = '0.' + addZeros('', shift - x.length) + x;
    }
    else {
        out =
            x.substr(0, x.length - shift) + '.' + x.substr(x.length - shift, x.length);
    }
    out = trimEnd(out);
    if (isNegative) {
        out = `-${out}`;
    }
    return out;
}
function addZeros(val, numZeros) {
    let out = val;
    for (let n = 0; n < numZeros; n++) {
        out += '0';
    }
    return out;
}
function cropHex(x) {
    return x.replace('0x', '');
}
// Takes two floating point (base 10) numbers and finds the multiplier needed to make them both
// operable as a integer
function floatShifts(xStart, yStart, moreShift) {
    let x = toBns(xStart);
    let y = toBns(yStart);
    let xPos = x.indexOf('.');
    let yPos = y.indexOf('.');
    const xHex = isHex(x);
    const yHex = isHex(y);
    if (xPos !== -1) {
        // Remove trailing zeros
        x = trimEnd(x);
        xPos = x.indexOf('.');
    }
    if (yPos !== -1) {
        // Remove trailing zeros
        y = trimEnd(y);
        yPos = y.indexOf('.');
    }
    if (xPos !== -1 || yPos !== -1 || typeof moreShift === 'number') {
        if (xHex || yHex) {
            throw new Error('Cannot operate on base16 float values');
        }
        let xShift = 0;
        let yShift = 0;
        if (xPos !== -1) {
            xShift = x.length - xPos - 1;
        }
        if (yPos !== -1) {
            yShift = y.length - yPos - 1;
        }
        const shift = xShift > yShift ? xShift : yShift;
        let moreS = 0;
        if (typeof moreShift === 'number') {
            moreS = moreShift;
        }
        x = addZeros(x.replace('.', ''), shift + moreS - xShift);
        y = addZeros(y.replace('.', ''), shift - yShift);
        const out = { x, y, shift };
        return out;
    }
    else {
        // Both x and y are int and need no float conversion
        const out = {
            x,
            y,
            shift: 0,
        };
        return out;
    }
}
function isHex(x1) {
    const x = toBns(x1);
    if (x.startsWith('0x') ||
        x.startsWith('-0x') ||
        x.toLowerCase().includes('a') ||
        x.toLowerCase().includes('b') ||
        x.toLowerCase().includes('c') ||
        x.toLowerCase().includes('d') ||
        x.toLowerCase().includes('e') ||
        x.toLowerCase().includes('f')) {
        return true;
    }
    else {
        return false;
    }
}
function precisionAdjust(type, x1, precision = 0) {
    var _a, _b;
    let x = toBns(x1);
    let negative = false;
    let out = '';
    if (x.includes('-')) {
        negative = true;
        // Remove any leading '-' signs
        x = x.replace(/^-+/, '');
    }
    const [whole, decimal = ''] = x.split('.');
    // Number of decimal places number has
    const decimalPos = x.indexOf('.');
    const checkIndex = decimalPos !== -1 ? decimalPos - precision : x.length - precision;
    const combined = whole + decimal;
    if (type === 'round') {
        const checkValue = (_a = combined[checkIndex]) !== null && _a !== void 0 ? _a : '0';
        if (gte(checkValue, '5')) {
            type = 'ceil';
        }
        else {
            type = 'floor';
        }
    }
    // Zero out lower precision places
    const numZeros = Math.max(combined.length - checkIndex + 1, 0);
    const zeroString = new Array(numZeros).join('0');
    let outCombined = combined.substring(0, checkIndex) + zeroString;
    const bumpUp = gt(combined.substring(checkIndex), '0');
    if (type === 'ceil' && bumpUp) {
        const addValue = '1' + zeroString;
        // Leading zeros get removed after an `add`. Save the number of leading zeros
        // and add them back
        const leadingZeros = ((_b = outCombined.match(/^0+/)) !== null && _b !== void 0 ? _b : [''])[0].length;
        const leadingZerosString = new Array(leadingZeros).join('0');
        outCombined = leadingZerosString + add(outCombined, addValue);
    }
    const newDecimalPos = decimalPos + (outCombined.length - combined.length);
    // Add back the decimal
    if (decimalPos !== -1) {
        out =
            outCombined.substring(0, newDecimalPos) +
                '.' +
                outCombined.substring(newDecimalPos);
    }
    else {
        out = outCombined;
    }
    out = trimEnd(out);
    if (negative && out !== '0') {
        out = '-' + out;
    }
    return out;
}
function sciNotation(x1, exponent) {
    const magnitude = exponent >= 0
        ? '1' + '0'.repeat(Math.abs(exponent)) // 5.8e7 -> 58000000
        : `0.${'0'.repeat(Math.abs(exponent) - 1)}1`; // 5.8e-7 -> 0.00000058
    return mul(x1, magnitude);
}
// Remove starting and trailing zeros and decimal
function trimEnd(val) {
    // Remove starting zeros if there are any
    let out = val.replace(/^0+/, '');
    out = out.replace(/^\.+/, '0.');
    if (out.includes('.')) {
        // Remove trailing zeros
        out = out.replace(/0+$/, '');
        // Remove trailing "." if there is one
        out = out.replace(/\.+$/, '');
        if (out === '') {
            out = '0';
        }
    }
    if (out === '') {
        out = '0';
    }
    return out;
}
function validate(...args) {
    for (const arg of args) {
        if (arg.split('.').length - 1 > 1) {
            throw new Error(`Invalid number: more than one decimal point '${arg}'`);
        }
        if (arg.split('-').length - 1 > 1) {
            throw new Error(`Invalid number: more than one negative sign '${arg}'`);
        }
        if (!/^-?(?:0x[0-9A-Fa-f]+|(?:\d+(?:\.\d*)?|\.\d+))$/.test(arg)) {
            throw new Error(`Invalid number: non-number characters '${arg}'`);
        }
    }
}
//# sourceMappingURL=index.js.map